/*
* Basic IO-Warrior functions for information about IO-Warrior device
* It show the device name, serialnumer and version for each connected IO-Warrior.
*/

#include <stdio.h>
#include <string.h>
#include "iowkit.h"


void get_iow_name(ULONG pid, char* name)
{
    switch(pid)
    {
        case IOWKIT_PID_IOW24: strcpy(name, "IO-Warrior24"); break;
        case IOWKIT_PID_IOWPV1: strcpy(name, "IO-Warrior24PV"); break;
        case IOWKIT_PID_IOW40: strcpy(name, "IO-Warrior40"); break;
        case IOWKIT_PID_IOW56: strcpy(name, "IO-Warrior56"); break;
        case IOWKIT_PID_IOW28: strcpy(name, "IO-Warrior28"); break;
        case IOWKIT_PID_IOW100: strcpy(name, "IO-Warrior100"); break;
    }
}

int main(int argc, char *argv[])
{
    IOWKIT_HANDLE handle;                   //Main handle, first found IO-Warrior (not the same every time!)
    IOWKIT_HANDLE iows[IOWKIT_MAX_DEVICES]; //Array for IO-Warrior devices
    unsigned short sn[9];                   //Serialnumber
    char snt[9];                            //Serialnumber as string
    DWORD pid;                              //ProductID
    ULONG rev;                              //Firmware version
    char name[64];                          //Device name

    //Open IO-Warrior API, return the first IO-Warrior found
    handle = IowKitOpenDevice();    
    if (handle == NULL)
    {
        printf("Failed to open device, EXIT\n");
        return 0;
    }

    //Print out some information about the IO-Warrior
    
    // Get version of SDK/API/DLL
    printf("API: %s\n", IowKitVersion());   

    // Get number of IOWs in system
    ULONG numIows = IowKitGetNumDevs();   
    printf("%ld IOWs in system\n\n", numIows);

    // Get all IOW handles
    for (ULONG i = 0; i < numIows; i++)
    {
        //Clear name array
        memset(&name, 0, 64);

        // Get device handle and init object
        iows[i] = IowKitGetDeviceHandle(i + 1);

        // Get serial number
        IowKitGetSerialNumber(iows[i], sn);

        //Get product ID
        pid = IowKitGetProductId(iows[i]);

        //Store the serial number into a string
        for (int j = 0; j < 9; j++)
            snt[j] = sn[j];

        //Get firmware revision
        rev = IowKitGetRevision(iows[i]);
        get_iow_name(pid, name);

        printf("%ld PID %x (%s), S/N \"%s\", Rev %lX\n", i + 1, (unsigned int)pid, name, snt, rev);
    }

    // Close device (NEVER forget to close iowkit!)
    IowKitCloseDevice(handle);
    return 0;
}