/*
* Simple IO sample to write and read IO-Ports of IO-Warrior.
* Autodetect the IO-Warrior and write/read the IO-Ports.
* Only first found IO-Warrior will be used by default.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "iowkit.h"

void get_iow_name(ULONG pid, char* name)
{
    switch(pid)
    {
        case IOWKIT_PID_IOW24: strcpy(name, "IO-Warrior24"); break;
        case IOWKIT_PID_IOWPV1: strcpy(name, "IO-Warrior24PV"); break;
        case IOWKIT_PID_IOW40: strcpy(name, "IO-Warrior40"); break;
        case IOWKIT_PID_IOW56: strcpy(name, "IO-Warrior56"); break;
        case IOWKIT_PID_IOW28: strcpy(name, "IO-Warrior28"); break;
        case IOWKIT_PID_IOW100: strcpy(name, "IO-Warrior100"); break;
    }
}

//Clear buffers of iowkit if needed
void drain_buffers(IOWKIT_HANDLE handle)
{
  char dummy[64];
  int pid = 0;
  int readsize = 0;

  pid = (IowKitGetProductId(handle));
  switch (pid)
  {
  case IOWKIT_PRODUCT_ID_IOW40:
    readsize = IOWKIT40_IO_REPORT_SIZE;
    break;
  case IOWKIT_PRODUCT_ID_IOW24:
    readsize = IOWKIT24_IO_REPORT_SIZE;
    break;
  case IOWKIT_PRODUCT_ID_IOW56:
  case IOWKIT_PRODUCT_ID_IOW28:
  case IOWKIT_PRODUCT_ID_IOW100:
    readsize = IOWKIT56_IO_REPORT_SIZE;
    break;
  }
  while (IowKitReadNonBlocking(handle, IOW_PIPE_IO_PINS, dummy, readsize))
    ;
}

//Write on IO-Port with values (selected IO-Port based on Starterkits/Evalboards)
ULONG write_io(IOWKIT_HANDLE handle, BYTE value)
{
    IOWKIT24_IO_REPORT rep24;   //IO-Warrior24
    IOWKIT40_IO_REPORT rep40;   //IO-Warrior40
    IOWKIT56_IO_REPORT rep56;   //IO-Warrior56
    IOWKIT28_IO_REPORT rep28;   //IO-Warrior28

  // Init report
  switch (IowKitGetProductId(handle))
  {
    // Write simple value to IOW40
    case IOWKIT_PRODUCT_ID_IOW40:
        memset(&rep40, 0xff, IOWKIT40_IO_REPORT_SIZE);
        rep40.ReportID = 0;
        rep40.Bytes[3] = (UCHAR)value;
        return IowKitWrite(handle, IOW_PIPE_IO_PINS, (PCHAR)&rep40, IOWKIT40_IO_REPORT_SIZE) == IOWKIT40_IO_REPORT_SIZE;

    // Write simple value to IOW24
    case IOWKIT_PRODUCT_ID_IOW24:
        memset(&rep24, 0xff, IOWKIT24_IO_REPORT_SIZE);
        rep24.ReportID = 0;
        rep24.Bytes[0] = (UCHAR)value;
        return IowKitWrite(handle, IOW_PIPE_IO_PINS, (PCHAR)&rep24, IOWKIT24_IO_REPORT_SIZE) == IOWKIT24_IO_REPORT_SIZE;

    // Write simple value to IOW56
    case IOWKIT_PRODUCT_ID_IOW56:
        memset(&rep56, 0xff, IOWKIT56_IO_REPORT_SIZE);
        rep56.ReportID = 0;
        rep56.Bytes[6] = (UCHAR)value;
        return IowKitWrite(handle, IOW_PIPE_IO_PINS, (PCHAR)&rep56, IOWKIT56_IO_REPORT_SIZE) == IOWKIT56_IO_REPORT_SIZE;

    // Write simple value to IOW28
    case IOWKIT_PRODUCT_ID_IOW28:
        memset(&rep28, 0xff, IOWKIT28_IO_REPORT_SIZE);
        rep28.ReportID = 0;
        rep28.Bytes[2] = (UCHAR)value;
        return IowKitWrite(handle, IOW_PIPE_IO_PINS, (PCHAR)&rep28, IOWKIT28_IO_REPORT_SIZE) == IOWKIT28_IO_REPORT_SIZE;

    // Write simple value to IOW100
    case IOWKIT_PRODUCT_ID_IOW100:
        //not implemented yet
        return 0;
    default:
        return FALSE;
  }
}

//Read IO-Ports of IO-Warrior. Used IOWKIT56_IO_REPORT as storage, because all IO-Warrior IO will fit to this
//Return value returns count or readed bytes. IowKitRead() will wait until new states on IO-Ports.
ULONG read_io(IOWKIT_HANDLE handle, IOWKIT56_IO_REPORT *rep56)
{
    //Clear report
    memset(rep56, 0xFF, IOWKIT56_IO_REPORT_SIZE);

    switch (IowKitGetProductId(handle))
    {
    case IOWKIT_PRODUCT_ID_IOW40:
        return IowKitRead(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT40_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW24:
        return IowKitRead(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT24_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW56:
        return IowKitRead(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT56_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW28:
        return IowKitRead(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT28_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW100:
        //not implemented yet
    default:
        return 0;
    }
}

//Read IO-Ports of IO-Warrior. Used IOWKIT56_IO_REPORT as storage, because all IO-Warrior IO will fit to this
//Return value returns count or readed bytes. IowKitReadNonBlocking() return last readed IO-Ports without blocking
ULONG read_io_nonblock(IOWKIT_HANDLE handle, IOWKIT56_IO_REPORT *rep56)
{
    //Clear report
    memset(rep56, 0xFF, IOWKIT56_IO_REPORT_SIZE);

    switch (IowKitGetProductId(handle))
    {
    case IOWKIT_PRODUCT_ID_IOW40:
        return IowKitReadNonBlocking(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT40_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW24:
        return IowKitReadNonBlocking(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT24_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW56:
        return IowKitReadNonBlocking(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT56_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW28:
        return IowKitReadNonBlocking(handle, IOW_PIPE_IO_PINS, (PCHAR)rep56, IOWKIT28_IO_REPORT_SIZE);
    case IOWKIT_PRODUCT_ID_IOW100:
        //not implemented yet
    default:
        return 0;
    }
}


int main(int argc, char *argv[])
{
    IOWKIT_HANDLE handle;       //Device handle
    unsigned short sn[9];       //Serialnumber
    char snt[9];                //Serialnumber as string
    char name[64];              //Device name
    ULONG bits;                 //Write buffer
    ULONG rc;                   //Read count

    // Open device and use first founc device furhter
    handle = IowKitOpenDevice();    
    if (handle == NULL)
    {
        printf("Failed to open device, EXIT\n");
        return 0;
    }

    //Get product ID
    DWORD pid = IowKitGetProductId(handle);
    
    //Get serial number
    IowKitGetSerialNumber(handle, sn);

    //Store the serial number into a string
    for (int j = 0; j < 9; j++)
        snt[j] = sn[j];

    //Get firmware revision
    ULONG rev = IowKitGetRevision(handle);
    get_iow_name(pid, name);

    //Print out device information
    printf("PID %x (%s), S/N \"%s\", Rev %lX\n",(unsigned int)pid, name, snt, rev);

    //Write random bits on IO-Ports based on the count of IO-Ports
    printf("Write IO-Ports....\n");
    
    srand(time(NULL));
    for (int i = 0; i < 20; i++)
    {
        bits = rand();

        // Write to simple endpoint
        rc = write_io(handle, bits);

        // Sleep for 25ms
        usleep(25000);
    }

    //Set IO-Port to 0xFF to set pins as input for read
    write_io(handle, 0xFF);

    //Clear buffer of iowkit
    drain_buffers(handle);

    //Read IO-Ports (if you use a starterkit or evalboard press the button to change the state of the IO-Pin)
    //IOWKIT56_IO_REPORT used, because fits all IO-Ports from other IO-Warrior.
    IOWKIT56_IO_REPORT rep56;

    printf("Read IO-Ports (stops after 10 seconds). If you have connected a Starterkit/Evalboard press a button.\n");
    for (int i = 0; i < 100; i++)
    {
        memset(&rep56, 0xFF, IOWKIT56_IO_REPORT_SIZE);
        rc = read_io_nonblock(handle, &rep56);
        if (rc)
        {
            printf("ReadNonBlocking(): rc=%u ", (int)rc);

            for (int j = 0; j < (int)rc - 1; j++)
                printf("%02x ", rep56.Bytes[j]);

            printf("\n");
        }

        usleep(100000);
    }

    //Close iowkit
    IowKitCloseDevice(handle);

    return 0;
}